ValueValidator

Struct ValueValidator 

pub struct ValueValidator<'a> {
    value: Cow<'a, Token>,
    data: &'a Everything,
    validated: bool,
    max_severity: Severity,
}
A validator for one Token. The intended usage is that the block-level Validator wraps the Token in a ValueValidator, then you call one or more of the validation functions on it. If the ValueValidator goes out of scope without having been validated, it will report an error.

Calling multiple validation functions is only supported when starting with the maybe_ variants. Calling one of the definite validation methods will either accept the value or warn about it, and it’s considered validated after that.

Fields

value: Cow<'a, Token>

The value being validated

data: &'a Everything

A link to all the loaded and processed CK3 and mod files

validated: bool

Whether the value has been validated. If true, it means either the value was accepted as correct or it was warned about.

max_severity: Severity

Maximum severity of problems reported by this ValueValidator. Defaults to Error. This is intended to be set lower by validators for less-important items. As an exception, Fatal severity reports will still always be logged as Fatal.

Implementations

Source

impl<'a> ValueValidator<'a>

pub fn new(value: &'a Token, data: &'a Everything) -> Self

Construct a new ValueValidator for a &Token.

pub fn new_owned(value: Token, data: &'a Everything) -> Self

Construct a new ValueValidator for an owned Token.

pub fn data(&self) -> &Everything

pub fn set_max_severity(&mut self, max_severity: Severity)

Maximum severity of problems reported by this ValueValidator. Defaults to Error. This is intended to be set lower by validators for less-important items. As an exception, Fatal severity reports will still always be logged as Fatal.

fn value_validator(&self, token: Token) -> Self

Make a descendant validator from this one, usually for a sub-token.

pub fn value(&self) -> &Token

Access the value that this ValueValidator validates.

pub fn accept(&mut self)

Mark this value as valid, without placing any restrictions on it.

pub fn identifier(&mut self, kind: &'static str)

pub fn item(&mut self, itype: Item)

Expect the value to be the key of an itype item in the game database. The item is looked up and must exist.

pub fn icon(&mut self, define: &str, suffix: &str)

pub fn item_used_with_suffix(&mut self, itype: Item, sfx: &str)

Add the given suffix to the value and mark that as a used item, without doing any validation. This is used for very weakly required localization, for example, where no warning is warranted.

pub fn implied_localization_sc( &mut self, pfx: &str, sfx: &str, sc: &mut ScopeContext, )

Validate a localization whose key is derived from the value, in the given ScopeContext.

pub fn maybe_item(&mut self, itype: Item) -> bool

Check if the value is the key of an itype item the game database. The item is looked up, and if it exists then this validator is considered validated. Return whether the item exists.

pub fn maybe_prefix_item(&mut self, pfx: &str, itype: Item) -> bool

Check if the value is be the key of an itype item the game database, after removing the prefix pfx. The item is looked up, and if it exists then this validator is considered validated. Return whether the item exists.

pub fn dir_file(&mut self, path: &str)

Expect the value to be the name of a file under the directory given here.

pub fn split(&mut self, c: char) -> Vec<ValueValidator<'_>>

pub fn target(&mut self, sc: &mut ScopeContext, outscopes: Scopes)

Expect the value to be a (possibly single-element) scope chain which evaluates to a scope type in outscopes.

The value is evaluated in the scope context sc, so for example if the value does scope:actor but there is no named scope “actor” in the scope context, then a warning is emitted.

Also emits a warning if the value is simply “this”, because that is almost never correct.

pub fn target_ok_this(&mut self, sc: &mut ScopeContext, outscopes: Scopes)

Just like ValueValidator::target, but allows the value to be simply “this”. It is expected to be used judiciously in cases where “this” can be correct.

pub fn item_or_target( &mut self, sc: &mut ScopeContext, itype: Item, outscopes: Scopes, )

This is a combination of ValueValidator::item and ValueValidator::target. If the field is present and is not a known itype item, then it is evaluated as a target.

pub fn bool(&mut self)

Expect the value to be just yes or no.

pub fn maybe_bool(&mut self)

Allow the value to be just yes or no, but don’t mandate it.

pub fn integer(&mut self)

Expect the value to be an integer.

pub fn maybe_integer(&mut self)

Allow the value to be an integer, but don’t mandate it.

pub fn integer_range<R: RangeBounds<i64>>(&mut self, range: R)

Expect the value to be an integer between low and high (inclusive).

pub fn numeric(&mut self)

Expect the value to be a number with up to 5 decimals. (5 decimals is the limit accepted by the game engine in most contexts).

pub fn numeric_range<R: RangeBounds<f64>>(&mut self, range: R)

Expect the value to be a number with up to 5 decimals within the range provided. (5 decimals is the limit accepted by the game engine in most contexts).

pub fn precise_numeric(&mut self)

Expect the value to be a number with any number of decimals.

pub fn date(&mut self)

Expect the value to be a date. The format of dates is very flexible, from a single number (the year), to a year.month or year.month.day. No checking is done on the validity of the date as a date (so January 42nd is okay).

pub fn choice(&mut self, choices: &[&str])

Expect the value to be one of the listed strings in choices.

pub fn maybe_is(&mut self, s: &str) -> bool

Check if the value is equal to the given string. If it is, mark this value as validated. Return whether the value matched the string.

pub fn warn_unvalidated(&mut self)

Tells the ValueValidator to report a warning if the value is still unvalidated.